/*
 * This file is part of play-clojurescript.
 * 
 * Copyright (c) 2013 Joachim Hofer
 * All rights reserved.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package de.johoop.play.plugin.clojurescript

import sbt.PlayExceptions.AssetCompilationException
import sbt.IO
import sbt._
import java.io.File
import clojure.lang.RT
import clojure.lang.Keyword
import java.nio.charset.Charset
import PlayExceptions._
import play.core.jscompile.JavascriptCompiler

// TODO maybe can be made a lot faster by running the compilation in a CLJS repl server?
object ClojureScriptCompiler {
  def compile(src: File, options: Seq[String]): (String, Option[String], Seq[File]) = {
    try {
      // seems to be required so that Clojure finds its stuff
      Thread.currentThread setContextClassLoader getClass.getClassLoader
      
      // (require 'cljs.closure')
      RT.`var`("clojure.core", "require").invoke(clojure.lang.Symbol.intern("cljs.closure"));
      
      val (out, opts) = compileOptions(src)
      
      // (cljs.closure/build srcFile compileOptions)
      RT.`var`("cljs.closure", "build").invoke(src.absolutePath, opts)
      
      val jsFileContent = IO.read(out, Charset forName "UTF-8")
      
      // minify using the Javascript compiler of Play2
      // (an alternative would be to directly let the ClojureScript compiler minify)
      val minified = 
        try Some(JavascriptCompiler minify (jsFileContent, Some(src.name)))
        catch {
          case e: CompilationException => 
            e.printStackTrace
            None 
        }
      
      (jsFileContent, minified, Seq(src))
      
    } catch {
      case e: Exception =>
        e.printStackTrace
        throw new AssetCompilationException(Some(src), "Internal ClojureScript Compiler error (see logs)", None, None)
    }
  }
  
  // optimization options: :optimizations :none/:simple/:whitespace/:advanced
  // we're using ":optimizations :simple" here together with ":pretty-print true"
  // in order to produce a nicely formatted single-file result 
  private def compileOptions(src: File): (File, clojure.lang.IPersistentMap) = {
    val out = IO.createTemporaryDirectory
    val outputFile = out / (src.base + ".js")
    (outputFile, RT.map(
      Keyword intern "output-dir", out.absolutePath,
      Keyword intern "output-to", outputFile.absolutePath,
      Keyword intern "optimizations", Keyword intern "simple",
      Keyword intern "pretty-print", java.lang.Boolean.TRUE))
  }
}